# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/tsfast',
                'doc_host': 'https://daniel-om-weber.github.io',
                'git_url': 'https://github.com/daniel-om-weber/tsfast',
                'lib_path': 'tsfast'},
  'syms': { 'tsfast.basics': {},
            'tsfast.benchmark': {'tsfast.benchmark.create_dls_from_spec': ('benchmark.html#create_dls_from_spec', 'tsfast/benchmark.py')},
            'tsfast.data.block': { 'tsfast.data.block.ScalarBlock': ('00_data/block.html#scalarblock', 'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarBlock.__init__': ( '00_data/block.html#scalarblock.__init__',
                                                                               'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarBlock.from_hdf': ( '00_data/block.html#scalarblock.from_hdf',
                                                                               'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize': ('00_data/block.html#scalarnormalize', 'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize.__init__': ( '00_data/block.html#scalarnormalize.__init__',
                                                                                   'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize.decodes': ( '00_data/block.html#scalarnormalize.decodes',
                                                                                  'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize.encodes': ( '00_data/block.html#scalarnormalize.encodes',
                                                                                  'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize.from_stats': ( '00_data/block.html#scalarnormalize.from_stats',
                                                                                     'tsfast/data/block.py'),
                                   'tsfast.data.block.ScalarNormalize.setups': ( '00_data/block.html#scalarnormalize.setups',
                                                                                 'tsfast/data/block.py'),
                                   'tsfast.data.block.SequenceBlock': ('00_data/block.html#sequenceblock', 'tsfast/data/block.py'),
                                   'tsfast.data.block.SequenceBlock.__init__': ( '00_data/block.html#sequenceblock.__init__',
                                                                                 'tsfast/data/block.py'),
                                   'tsfast.data.block.SequenceBlock.from_hdf': ( '00_data/block.html#sequenceblock.from_hdf',
                                                                                 'tsfast/data/block.py'),
                                   'tsfast.data.block.SequenceBlock.from_numpy': ( '00_data/block.html#sequenceblock.from_numpy',
                                                                                   'tsfast/data/block.py'),
                                   'tsfast.data.block.pad_sequence': ('00_data/block.html#pad_sequence', 'tsfast/data/block.py')},
            'tsfast.data.core': { 'tsfast.data.core.CreateDict': ('00_data/core.html#createdict', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfApplyFuncSplit': ('00_data/core.html#dfapplyfuncsplit', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfDropClmExcept': ('00_data/core.html#dfdropclmexcept', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfFilterQuery': ('00_data/core.html#dffilterquery', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfHDFCreateWindows': ('00_data/core.html#dfhdfcreatewindows', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfHDFGetSeqLen': ('00_data/core.html#dfhdfgetseqlen', 'tsfast/data/core.py'),
                                  'tsfast.data.core.DfResamplingFactor': ('00_data/core.html#dfresamplingfactor', 'tsfast/data/core.py'),
                                  'tsfast.data.core.FilterClm': ('00_data/core.html#filterclm', 'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Scalars': ('00_data/core.html#hdf2scalars', 'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Scalars.__init__': ( '00_data/core.html#hdf2scalars.__init__',
                                                                             'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Scalars._extract_dict_scalars': ( '00_data/core.html#hdf2scalars._extract_dict_scalars',
                                                                                          'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Scalars.encodes': ('00_data/core.html#hdf2scalars.encodes', 'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Sequence': ('00_data/core.html#hdf2sequence', 'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Sequence.__init__': ( '00_data/core.html#hdf2sequence.__init__',
                                                                              'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Sequence._extract_dict_sequence': ( '00_data/core.html#hdf2sequence._extract_dict_sequence',
                                                                                            'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Sequence._hdf_extract_sequence': ( '00_data/core.html#hdf2sequence._hdf_extract_sequence',
                                                                                           'tsfast/data/core.py'),
                                  'tsfast.data.core.HDF2Sequence.encodes': ( '00_data/core.html#hdf2sequence.encodes',
                                                                             'tsfast/data/core.py'),
                                  'tsfast.data.core.Memoize': ('00_data/core.html#memoize', 'tsfast/data/core.py'),
                                  'tsfast.data.core.Memoize.__call__': ('00_data/core.html#memoize.__call__', 'tsfast/data/core.py'),
                                  'tsfast.data.core.Memoize.__init__': ('00_data/core.html#memoize.__init__', 'tsfast/data/core.py'),
                                  'tsfast.data.core.MemoizeMP': ('00_data/core.html#memoizemp', 'tsfast/data/core.py'),
                                  'tsfast.data.core.MemoizeMP.__call__': ('00_data/core.html#memoizemp.__call__', 'tsfast/data/core.py'),
                                  'tsfast.data.core.MemoizeMP.__del__': ('00_data/core.html#memoizemp.__del__', 'tsfast/data/core.py'),
                                  'tsfast.data.core.MemoizeMP.__init__': ('00_data/core.html#memoizemp.__init__', 'tsfast/data/core.py'),
                                  'tsfast.data.core.MemoizeMP.cleanup_shared_memory': ( '00_data/core.html#memoizemp.cleanup_shared_memory',
                                                                                        'tsfast/data/core.py'),
                                  'tsfast.data.core.ScalarSequenceElement': ( '00_data/core.html#scalarsequenceelement',
                                                                              'tsfast/data/core.py'),
                                  'tsfast.data.core.ScalarSequenceElement.__init__': ( '00_data/core.html#scalarsequenceelement.__init__',
                                                                                       'tsfast/data/core.py'),
                                  'tsfast.data.core.ScalarSequenceElement.encodes': ( '00_data/core.html#scalarsequenceelement.encodes',
                                                                                      'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorScalars': ('00_data/core.html#tensorscalars', 'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorScalars.from_hdf': ( '00_data/core.html#tensorscalars.from_hdf',
                                                                               'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorScalarsInput': ('00_data/core.html#tensorscalarsinput', 'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorScalarsOutput': ('00_data/core.html#tensorscalarsoutput', 'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorSequences': ('00_data/core.html#tensorsequences', 'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorSequences.from_hdf': ( '00_data/core.html#tensorsequences.from_hdf',
                                                                                 'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorSequences.show': ( '00_data/core.html#tensorsequences.show',
                                                                             'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorSequencesInput': ( '00_data/core.html#tensorsequencesinput',
                                                                             'tsfast/data/core.py'),
                                  'tsfast.data.core.TensorSequencesOutput': ( '00_data/core.html#tensorsequencesoutput',
                                                                              'tsfast/data/core.py'),
                                  'tsfast.data.core.ValidClmContains': ('00_data/core.html#validclmcontains', 'tsfast/data/core.py'),
                                  'tsfast.data.core.ValidClmIs': ('00_data/core.html#validclmis', 'tsfast/data/core.py'),
                                  'tsfast.data.core.apply_df_tfms': ('00_data/core.html#apply_df_tfms', 'tsfast/data/core.py'),
                                  'tsfast.data.core.calc_shift_offsets': ('00_data/core.html#calc_shift_offsets', 'tsfast/data/core.py'),
                                  'tsfast.data.core.count_parameters': ('00_data/core.html#count_parameters', 'tsfast/data/core.py'),
                                  'tsfast.data.core.df_get_hdf_seq_len': ('00_data/core.html#df_get_hdf_seq_len', 'tsfast/data/core.py'),
                                  'tsfast.data.core.downsample_mean': ('00_data/core.html#downsample_mean', 'tsfast/data/core.py'),
                                  'tsfast.data.core.get_hdf_files': ('00_data/core.html#get_hdf_files', 'tsfast/data/core.py'),
                                  'tsfast.data.core.get_hdf_seq_len': ('00_data/core.html#get_hdf_seq_len', 'tsfast/data/core.py'),
                                  'tsfast.data.core.hdf2scalars': ('00_data/core.html#hdf2scalars', 'tsfast/data/core.py'),
                                  'tsfast.data.core.hdf_extract_sequence': ( '00_data/core.html#hdf_extract_sequence',
                                                                             'tsfast/data/core.py'),
                                  'tsfast.data.core.obj_in_lst': ('00_data/core.html#obj_in_lst', 'tsfast/data/core.py'),
                                  'tsfast.data.core.plot_seqs_multi_figures': ( '00_data/core.html#plot_seqs_multi_figures',
                                                                                'tsfast/data/core.py'),
                                  'tsfast.data.core.plot_seqs_single_figure': ( '00_data/core.html#plot_seqs_single_figure',
                                                                                'tsfast/data/core.py'),
                                  'tsfast.data.core.plot_sequence': ('00_data/core.html#plot_sequence', 'tsfast/data/core.py'),
                                  'tsfast.data.core.resample_interp': ('00_data/core.html#resample_interp', 'tsfast/data/core.py'),
                                  'tsfast.data.core.running_mean': ('00_data/core.html#running_mean', 'tsfast/data/core.py'),
                                  'tsfast.data.core.show_batch': ('00_data/core.html#show_batch', 'tsfast/data/core.py'),
                                  'tsfast.data.core.show_results': ('00_data/core.html#show_results', 'tsfast/data/core.py'),
                                  'tsfast.data.core.toTensorSequencesInput': ( '00_data/core.html#totensorsequencesinput',
                                                                               'tsfast/data/core.py'),
                                  'tsfast.data.core.toTensorSequencesOutput': ( '00_data/core.html#totensorsequencesoutput',
                                                                                'tsfast/data/core.py')},
            'tsfast.data.loader': { 'tsfast.data.loader.BatchLimit_Factory': ( '00_data/loader.html#batchlimit_factory',
                                                                               'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl': ('00_data/loader.html#tbpttdl', 'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.__init__': ( '00_data/loader.html#tbpttdl.__init__',
                                                                             'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.__iter__': ( '00_data/loader.html#tbpttdl.__iter__',
                                                                             'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.__len__': ('00_data/loader.html#tbpttdl.__len__', 'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl._next_worker': ( '00_data/loader.html#tbpttdl._next_worker',
                                                                                 'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl._tbptt_generator': ( '00_data/loader.html#tbpttdl._tbptt_generator',
                                                                                     'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.create_batches': ( '00_data/loader.html#tbpttdl.create_batches',
                                                                                   'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.n_sub_seq': ( '00_data/loader.html#tbpttdl.n_sub_seq',
                                                                              'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttDl.sample': ('00_data/loader.html#tbpttdl.sample', 'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttResetCB': ('00_data/loader.html#tbpttresetcb', 'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttResetCB.after_fit': ( '00_data/loader.html#tbpttresetcb.after_fit',
                                                                                   'tsfast/data/loader.py'),
                                    'tsfast.data.loader.TbpttResetCB.before_batch': ( '00_data/loader.html#tbpttresetcb.before_batch',
                                                                                      'tsfast/data/loader.py'),
                                    'tsfast.data.loader.WeightedDL_Factory': ( '00_data/loader.html#weighteddl_factory',
                                                                               'tsfast/data/loader.py'),
                                    'tsfast.data.loader.reset_model_state': ( '00_data/loader.html#reset_model_state',
                                                                              'tsfast/data/loader.py'),
                                    'tsfast.data.loader.uniform_p_of_category': ( '00_data/loader.html#uniform_p_of_category',
                                                                                  'tsfast/data/loader.py'),
                                    'tsfast.data.loader.uniform_p_of_float': ( '00_data/loader.html#uniform_p_of_float',
                                                                               'tsfast/data/loader.py'),
                                    'tsfast.data.loader.uniform_p_of_float_with_gaps': ( '00_data/loader.html#uniform_p_of_float_with_gaps',
                                                                                         'tsfast/data/loader.py')},
            'tsfast.data.split': { 'tsfast.data.split.ApplyToDict': ('00_data/split.html#applytodict', 'tsfast/data/split.py'),
                                   'tsfast.data.split.ParentSplitter': ('00_data/split.html#parentsplitter', 'tsfast/data/split.py'),
                                   'tsfast.data.split.PercentageSplitter': ( '00_data/split.html#percentagesplitter',
                                                                             'tsfast/data/split.py'),
                                   'tsfast.data.split._parent_idxs': ('00_data/split.html#_parent_idxs', 'tsfast/data/split.py')},
            'tsfast.data.transforms': { 'tsfast.data.transforms.SeqBiasInjection': ( '00_data/transforms.html#seqbiasinjection',
                                                                                     'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqBiasInjection.__init__': ( '00_data/transforms.html#seqbiasinjection.__init__',
                                                                                              'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqBiasInjection.encodes': ( '00_data/transforms.html#seqbiasinjection.encodes',
                                                                                             'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection': ( '00_data/transforms.html#seqnoiseinjection',
                                                                                      'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection.__init__': ( '00_data/transforms.html#seqnoiseinjection.__init__',
                                                                                               'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection.encodes': ( '00_data/transforms.html#seqnoiseinjection.encodes',
                                                                                              'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Grouped': ( '00_data/transforms.html#seqnoiseinjection_grouped',
                                                                                              'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Grouped.__init__': ( '00_data/transforms.html#seqnoiseinjection_grouped.__init__',
                                                                                                       'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Grouped.encodes': ( '00_data/transforms.html#seqnoiseinjection_grouped.encodes',
                                                                                                      'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Varying': ( '00_data/transforms.html#seqnoiseinjection_varying',
                                                                                              'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Varying.__init__': ( '00_data/transforms.html#seqnoiseinjection_varying.__init__',
                                                                                                       'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqNoiseInjection_Varying.encodes': ( '00_data/transforms.html#seqnoiseinjection_varying.encodes',
                                                                                                      'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqSlice': ( '00_data/transforms.html#seqslice',
                                                                             'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqSlice.__init__': ( '00_data/transforms.html#seqslice.__init__',
                                                                                      'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.SeqSlice.encodes': ( '00_data/transforms.html#seqslice.encodes',
                                                                                     'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.decodes': ('00_data/transforms.html#decodes', 'tsfast/data/transforms.py'),
                                        'tsfast.data.transforms.encodes': ('00_data/transforms.html#encodes', 'tsfast/data/transforms.py')},
            'tsfast.datasets.benchmark': { 'tsfast.datasets.benchmark.create_dls_from_spec': ( '01_datasets/benchmark.html#create_dls_from_spec',
                                                                                               'tsfast/datasets/benchmark.py')},
            'tsfast.datasets.core': { 'tsfast.datasets.core.clean_default_dataset_path': ( '01_datasets/core.html#clean_default_dataset_path',
                                                                                           'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.create_dls': ('01_datasets/core.html#create_dls', 'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.create_dls_downl': ( '01_datasets/core.html#create_dls_downl',
                                                                                 'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.dict_file_load': ( '01_datasets/core.html#dict_file_load',
                                                                               'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.dict_file_save': ( '01_datasets/core.html#dict_file_save',
                                                                               'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.extract_mean_std_from_dataset': ( '01_datasets/core.html#extract_mean_std_from_dataset',
                                                                                              'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.extract_mean_std_from_dls': ( '01_datasets/core.html#extract_mean_std_from_dls',
                                                                                          'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.extract_mean_std_from_hdffiles': ( '01_datasets/core.html#extract_mean_std_from_hdffiles',
                                                                                               'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.get_dataset_path': ( '01_datasets/core.html#get_dataset_path',
                                                                                 'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.get_default_dataset_path': ( '01_datasets/core.html#get_default_dataset_path',
                                                                                         'tsfast/datasets/core.py'),
                                      'tsfast.datasets.core.is_dataset_directory': ( '01_datasets/core.html#is_dataset_directory',
                                                                                     'tsfast/datasets/core.py')},
            'tsfast.inference.core': { 'tsfast.inference.core.InferenceWrapper': ( '05_inference/core.html#inferencewrapper',
                                                                                   'tsfast/inference/core.py'),
                                       'tsfast.inference.core.InferenceWrapper.__call__': ( '05_inference/core.html#inferencewrapper.__call__',
                                                                                            'tsfast/inference/core.py'),
                                       'tsfast.inference.core.InferenceWrapper.__init__': ( '05_inference/core.html#inferencewrapper.__init__',
                                                                                            'tsfast/inference/core.py'),
                                       'tsfast.inference.core.InferenceWrapper._adjust_seq_len': ( '05_inference/core.html#inferencewrapper._adjust_seq_len',
                                                                                                   'tsfast/inference/core.py'),
                                       'tsfast.inference.core.InferenceWrapper._prepare_tensor': ( '05_inference/core.html#inferencewrapper._prepare_tensor',
                                                                                                   'tsfast/inference/core.py'),
                                       'tsfast.inference.core.InferenceWrapper.inference': ( '05_inference/core.html#inferencewrapper.inference',
                                                                                             'tsfast/inference/core.py')},
            'tsfast.learner.callbacks': { 'tsfast.learner.callbacks.ARInitCB': ( '02_learner/callbacks.html#arinitcb',
                                                                                 'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.ARInitCB.before_batch': ( '02_learner/callbacks.html#arinitcb.before_batch',
                                                                                              'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.BatchLossFilter': ( '02_learner/callbacks.html#batchlossfilter',
                                                                                        'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.BatchLossFilter.__init__': ( '02_learner/callbacks.html#batchlossfilter.__init__',
                                                                                                 'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.BatchLossFilter.after_pred': ( '02_learner/callbacks.html#batchlossfilter.after_pred',
                                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_AddLoss': ( '02_learner/callbacks.html#cb_addloss',
                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_AddLoss.__init__': ( '02_learner/callbacks.html#cb_addloss.__init__',
                                                                                            'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_AddLoss.after_loss': ( '02_learner/callbacks.html#cb_addloss.after_loss',
                                                                                              'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_PlotGradient': ( '02_learner/callbacks.html#cb_plotgradient',
                                                                                        'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_PlotGradient.__init__': ( '02_learner/callbacks.html#cb_plotgradient.__init__',
                                                                                                 'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_PlotGradient.after_backward': ( '02_learner/callbacks.html#cb_plotgradient.after_backward',
                                                                                                       'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_PlotGradient.begin_fit': ( '02_learner/callbacks.html#cb_plotgradient.begin_fit',
                                                                                                  'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_TruncateSequence': ( '02_learner/callbacks.html#cb_truncatesequence',
                                                                                            'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_TruncateSequence.__init__': ( '02_learner/callbacks.html#cb_truncatesequence.__init__',
                                                                                                     'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CB_TruncateSequence.before_batch': ( '02_learner/callbacks.html#cb_truncatesequence.before_batch',
                                                                                                         'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CancelNaNCallback': ( '02_learner/callbacks.html#cancelnancallback',
                                                                                          'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.CancelNaNCallback.after_loss': ( '02_learner/callbacks.html#cancelnancallback.after_loss',
                                                                                                     'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientBatchFiltering': ( '02_learner/callbacks.html#gradientbatchfiltering',
                                                                                               'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientBatchFiltering.__init__': ( '02_learner/callbacks.html#gradientbatchfiltering.__init__',
                                                                                                        'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientBatchFiltering.before_step': ( '02_learner/callbacks.html#gradientbatchfiltering.before_step',
                                                                                                           'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientClipping': ( '02_learner/callbacks.html#gradientclipping',
                                                                                         'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientClipping.__init__': ( '02_learner/callbacks.html#gradientclipping.__init__',
                                                                                                  'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientClipping.after_backward': ( '02_learner/callbacks.html#gradientclipping.after_backward',
                                                                                                        'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientNormPrint': ( '02_learner/callbacks.html#gradientnormprint',
                                                                                          'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.GradientNormPrint.before_step': ( '02_learner/callbacks.html#gradientnormprint.before_step',
                                                                                                      'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.SkipFirstNCallback': ( '02_learner/callbacks.html#skipfirstncallback',
                                                                                           'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.SkipFirstNCallback.__init__': ( '02_learner/callbacks.html#skipfirstncallback.__init__',
                                                                                                    'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.SkipFirstNCallback.after_pred': ( '02_learner/callbacks.html#skipfirstncallback.after_pred',
                                                                                                      'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.SkipNaNCallback': ( '02_learner/callbacks.html#skipnancallback',
                                                                                        'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.SkipNaNCallback.after_loss': ( '02_learner/callbacks.html#skipnancallback.after_loss',
                                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.TimeSeriesRegularizer': ( '02_learner/callbacks.html#timeseriesregularizer',
                                                                                              'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.TimeSeriesRegularizer.__init__': ( '02_learner/callbacks.html#timeseriesregularizer.__init__',
                                                                                                       'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.TimeSeriesRegularizer.after_loss': ( '02_learner/callbacks.html#timeseriesregularizer.after_loss',
                                                                                                         'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.TimeSeriesRegularizer.hook': ( '02_learner/callbacks.html#timeseriesregularizer.hook',
                                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.VarySeqLen': ( '02_learner/callbacks.html#varyseqlen',
                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.VarySeqLen.__init__': ( '02_learner/callbacks.html#varyseqlen.__init__',
                                                                                            'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.VarySeqLen.before_batch': ( '02_learner/callbacks.html#varyseqlen.before_batch',
                                                                                                'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.WeightClipping': ( '02_learner/callbacks.html#weightclipping',
                                                                                       'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.WeightClipping.__init__': ( '02_learner/callbacks.html#weightclipping.__init__',
                                                                                                'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.WeightClipping.after_batch': ( '02_learner/callbacks.html#weightclipping.after_batch',
                                                                                                   'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.plot_grad_flow': ( '02_learner/callbacks.html#plot_grad_flow',
                                                                                       'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.sched_lin_p': ( '02_learner/callbacks.html#sched_lin_p',
                                                                                    'tsfast/learner/callbacks.py'),
                                          'tsfast.learner.callbacks.sched_ramp': ( '02_learner/callbacks.html#sched_ramp',
                                                                                   'tsfast/learner/callbacks.py')},
            'tsfast.learner.learner': { 'tsfast.learner.learner.AR_RNNLearner': ( '02_learner/learner.html#ar_rnnlearner',
                                                                                  'tsfast/learner/learner.py'),
                                        'tsfast.learner.learner.AR_TCNLearner': ( '02_learner/learner.html#ar_tcnlearner',
                                                                                  'tsfast/learner/learner.py'),
                                        'tsfast.learner.learner.CRNNLearner': ( '02_learner/learner.html#crnnlearner',
                                                                                'tsfast/learner/learner.py'),
                                        'tsfast.learner.learner.RNNLearner': ( '02_learner/learner.html#rnnlearner',
                                                                               'tsfast/learner/learner.py'),
                                        'tsfast.learner.learner.TCNLearner': ( '02_learner/learner.html#tcnlearner',
                                                                               'tsfast/learner/learner.py'),
                                        'tsfast.learner.learner.get_inp_out_size': ( '02_learner/learner.html#get_inp_out_size',
                                                                                     'tsfast/learner/learner.py')},
            'tsfast.learner.losses': { 'tsfast.learner.losses.CutLoss': ('02_learner/losses.html#cutloss', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.RandSeqLenLoss': ( '02_learner/losses.html#randseqlenloss',
                                                                                 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.SkipNLoss': ('02_learner/losses.html#skipnloss', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.cos_sim_loss': ( '02_learner/losses.html#cos_sim_loss',
                                                                               'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.cos_sim_loss_pow': ( '02_learner/losses.html#cos_sim_loss_pow',
                                                                                   'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.float64_func': ( '02_learner/losses.html#float64_func',
                                                                               'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.fun_rmse': ('02_learner/losses.html#fun_rmse', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.ignore_nan': ( '02_learner/losses.html#ignore_nan',
                                                                             'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.mean_vaf': ('02_learner/losses.html#mean_vaf', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.nrmse': ('02_learner/losses.html#nrmse', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.nrmse_std': ('02_learner/losses.html#nrmse_std', 'tsfast/learner/losses.py'),
                                       'tsfast.learner.losses.weighted_mae': ( '02_learner/losses.html#weighted_mae',
                                                                               'tsfast/learner/losses.py')},
            'tsfast.models.cnn': { 'tsfast.models.cnn.CConv1D': ('03_models/cnn.html#cconv1d', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CNN': ('03_models/cnn.html#cnn', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CNN.__init__': ('03_models/cnn.html#cnn.__init__', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CNN.forward': ('03_models/cnn.html#cnn.forward', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CRNN': ('03_models/cnn.html#crnn', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CRNN.__init__': ('03_models/cnn.html#crnn.__init__', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CRNN.forward': ('03_models/cnn.html#crnn.forward', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CausalConv1d': ('03_models/cnn.html#causalconv1d', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CausalConv1d.__init__': ( '03_models/cnn.html#causalconv1d.__init__',
                                                                                'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CausalConv1d.forward': ( '03_models/cnn.html#causalconv1d.forward',
                                                                               'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.CausalConv1d.reset_state': ( '03_models/cnn.html#causalconv1d.reset_state',
                                                                                   'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.Conv1D': ('03_models/cnn.html#conv1d', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateCRNN': ('03_models/cnn.html#seperatecrnn', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateCRNN.__init__': ( '03_models/cnn.html#seperatecrnn.__init__',
                                                                                'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateCRNN.forward': ( '03_models/cnn.html#seperatecrnn.forward',
                                                                               'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateTCN': ('03_models/cnn.html#seperatetcn', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateTCN.__init__': ( '03_models/cnn.html#seperatetcn.__init__',
                                                                               'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateTCN.forward': ( '03_models/cnn.html#seperatetcn.forward',
                                                                              'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.SeperateTCN.reset_state': ( '03_models/cnn.html#seperatetcn.reset_state',
                                                                                  'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN': ('03_models/cnn.html#tcn', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN.__init__': ('03_models/cnn.html#tcn.__init__', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN.forward': ('03_models/cnn.html#tcn.forward', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN_Block': ('03_models/cnn.html#tcn_block', 'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN_Block.__init__': ( '03_models/cnn.html#tcn_block.__init__',
                                                                             'tsfast/models/cnn.py'),
                                   'tsfast.models.cnn.TCN_Block.forward': ('03_models/cnn.html#tcn_block.forward', 'tsfast/models/cnn.py')},
            'tsfast.models.layers': { 'tsfast.models.layers.AR_Model': ('03_models/layers.html#ar_model', 'tsfast/models/layers.py'),
                                      'tsfast.models.layers.AR_Model.__init__': ( '03_models/layers.html#ar_model.__init__',
                                                                                  'tsfast/models/layers.py'),
                                      'tsfast.models.layers.AR_Model.forward': ( '03_models/layers.html#ar_model.forward',
                                                                                 'tsfast/models/layers.py'),
                                      'tsfast.models.layers.AR_Model.init_normalize': ( '03_models/layers.html#ar_model.init_normalize',
                                                                                        'tsfast/models/layers.py'),
                                      'tsfast.models.layers.AR_Model.init_normalize_values': ( '03_models/layers.html#ar_model.init_normalize_values',
                                                                                               'tsfast/models/layers.py'),
                                      'tsfast.models.layers.AR_Model.reset_state': ( '03_models/layers.html#ar_model.reset_state',
                                                                                     'tsfast/models/layers.py'),
                                      'tsfast.models.layers.BatchNorm_1D_Stateful': ( '03_models/layers.html#batchnorm_1d_stateful',
                                                                                      'tsfast/models/layers.py'),
                                      'tsfast.models.layers.BatchNorm_1D_Stateful.__init__': ( '03_models/layers.html#batchnorm_1d_stateful.__init__',
                                                                                               'tsfast/models/layers.py'),
                                      'tsfast.models.layers.BatchNorm_1D_Stateful.forward': ( '03_models/layers.html#batchnorm_1d_stateful.forward',
                                                                                              'tsfast/models/layers.py'),
                                      'tsfast.models.layers.BatchNorm_1D_Stateful.reset_parameters': ( '03_models/layers.html#batchnorm_1d_stateful.reset_parameters',
                                                                                                       'tsfast/models/layers.py'),
                                      'tsfast.models.layers.BatchNorm_1D_Stateful.reset_state': ( '03_models/layers.html#batchnorm_1d_stateful.reset_state',
                                                                                                  'tsfast/models/layers.py'),
                                      'tsfast.models.layers.NormalizedModel': ( '03_models/layers.html#normalizedmodel',
                                                                                'tsfast/models/layers.py'),
                                      'tsfast.models.layers.NormalizedModel.__init__': ( '03_models/layers.html#normalizedmodel.__init__',
                                                                                         'tsfast/models/layers.py'),
                                      'tsfast.models.layers.NormalizedModel.forward': ( '03_models/layers.html#normalizedmodel.forward',
                                                                                        'tsfast/models/layers.py'),
                                      'tsfast.models.layers.NormalizedModel.from_dls': ( '03_models/layers.html#normalizedmodel.from_dls',
                                                                                         'tsfast/models/layers.py'),
                                      'tsfast.models.layers.Normalizer1D': ( '03_models/layers.html#normalizer1d',
                                                                             'tsfast/models/layers.py'),
                                      'tsfast.models.layers.Normalizer1D.__init__': ( '03_models/layers.html#normalizer1d.__init__',
                                                                                      'tsfast/models/layers.py'),
                                      'tsfast.models.layers.Normalizer1D.normalize': ( '03_models/layers.html#normalizer1d.normalize',
                                                                                       'tsfast/models/layers.py'),
                                      'tsfast.models.layers.Normalizer1D.unnormalize': ( '03_models/layers.html#normalizer1d.unnormalize',
                                                                                         'tsfast/models/layers.py'),
                                      'tsfast.models.layers.SeqLinear': ('03_models/layers.html#seqlinear', 'tsfast/models/layers.py'),
                                      'tsfast.models.layers.SeqLinear.__init__': ( '03_models/layers.html#seqlinear.__init__',
                                                                                   'tsfast/models/layers.py'),
                                      'tsfast.models.layers.SeqLinear.forward': ( '03_models/layers.html#seqlinear.forward',
                                                                                  'tsfast/models/layers.py')},
            'tsfast.models.rnn': { 'tsfast.models.rnn.DenseBlock_RNN': ('03_models/rnn.html#denseblock_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseBlock_RNN.__init__': ( '03_models/rnn.html#denseblock_rnn.__init__',
                                                                                  'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseLayer_RNN': ('03_models/rnn.html#denselayer_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseLayer_RNN.__init__': ( '03_models/rnn.html#denselayer_rnn.__init__',
                                                                                  'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseLayer_RNN.forward': ( '03_models/rnn.html#denselayer_rnn.forward',
                                                                                 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseNet_RNN': ('03_models/rnn.html#densenet_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.DenseNet_RNN.__init__': ( '03_models/rnn.html#densenet_rnn.__init__',
                                                                                'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN': ('03_models/rnn.html#rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN.__init__': ('03_models/rnn.html#rnn.__init__', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN._get_hidden': ('03_models/rnn.html#rnn._get_hidden', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN._one_rnn': ('03_models/rnn.html#rnn._one_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN.forward': ('03_models/rnn.html#rnn.forward', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.RNN.reset_state': ('03_models/rnn.html#rnn.reset_state', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.ResidualBlock_RNN': ('03_models/rnn.html#residualblock_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.ResidualBlock_RNN.__init__': ( '03_models/rnn.html#residualblock_rnn.__init__',
                                                                                     'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.ResidualBlock_RNN.forward': ( '03_models/rnn.html#residualblock_rnn.forward',
                                                                                    'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SeperateRNN': ('03_models/rnn.html#seperaternn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SeperateRNN.__init__': ( '03_models/rnn.html#seperaternn.__init__',
                                                                               'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SeperateRNN.forward': ( '03_models/rnn.html#seperaternn.forward',
                                                                              'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.Sequential_RNN': ('03_models/rnn.html#sequential_rnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.Sequential_RNN.forward': ( '03_models/rnn.html#sequential_rnn.forward',
                                                                                 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SimpleRNN': ('03_models/rnn.html#simplernn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SimpleRNN.__init__': ( '03_models/rnn.html#simplernn.__init__',
                                                                             'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SimpleRNN.forward': ('03_models/rnn.html#simplernn.forward', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SimpleResidualRNN': ('03_models/rnn.html#simpleresidualrnn', 'tsfast/models/rnn.py'),
                                   'tsfast.models.rnn.SimpleResidualRNN.__init__': ( '03_models/rnn.html#simpleresidualrnn.__init__',
                                                                                     'tsfast/models/rnn.py')},
            'tsfast.prediction.ar': { 'tsfast.prediction.ar.ARProg': ('04_prediction/ar.html#arprog', 'tsfast/prediction/ar.py'),
                                      'tsfast.prediction.ar.ARProg.__init__': ( '04_prediction/ar.html#arprog.__init__',
                                                                                'tsfast/prediction/ar.py'),
                                      'tsfast.prediction.ar.ARProg.forward': ( '04_prediction/ar.html#arprog.forward',
                                                                               'tsfast/prediction/ar.py')},
            'tsfast.prediction.core': { 'tsfast.prediction.core.PredictionCallback': ( '04_prediction/core.html#predictioncallback',
                                                                                       'tsfast/prediction/core.py'),
                                        'tsfast.prediction.core.PredictionCallback.__init__': ( '04_prediction/core.html#predictioncallback.__init__',
                                                                                                'tsfast/prediction/core.py'),
                                        'tsfast.prediction.core.PredictionCallback.before_batch': ( '04_prediction/core.html#predictioncallback.before_batch',
                                                                                                    'tsfast/prediction/core.py'),
                                        'tsfast.prediction.core.PredictionCallback.init_normalize': ( '04_prediction/core.html#predictioncallback.init_normalize',
                                                                                                      'tsfast/prediction/core.py'),
                                        'tsfast.prediction.core.PredictionCallback.init_normalize_values': ( '04_prediction/core.html#predictioncallback.init_normalize_values',
                                                                                                             'tsfast/prediction/core.py')},
            'tsfast.prediction.fransys': { 'tsfast.prediction.fransys.ARProg_Init': ( '04_prediction/fransys.html#arprog_init',
                                                                                      'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.ARProg_Init.__init__': ( '04_prediction/fransys.html#arprog_init.__init__',
                                                                                               'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.ARProg_Init.forward': ( '04_prediction/fransys.html#arprog_init.forward',
                                                                                              'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.DiagLSTM': ( '04_prediction/fransys.html#diaglstm',
                                                                                   'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.DiagLSTM.__init__': ( '04_prediction/fransys.html#diaglstm.__init__',
                                                                                            'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.DiagLSTM._get_hidden': ( '04_prediction/fransys.html#diaglstm._get_hidden',
                                                                                               'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.DiagLSTM.forward': ( '04_prediction/fransys.html#diaglstm.forward',
                                                                                           'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.DiagLSTM.output_to_hidden': ( '04_prediction/fransys.html#diaglstm.output_to_hidden',
                                                                                                    'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN': ( '04_prediction/fransys.html#diag_rnn',
                                                                                   'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN.__init__': ( '04_prediction/fransys.html#diag_rnn.__init__',
                                                                                            'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN._get_hidden': ( '04_prediction/fransys.html#diag_rnn._get_hidden',
                                                                                               'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN.forward': ( '04_prediction/fransys.html#diag_rnn.forward',
                                                                                           'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN.output_to_hidden': ( '04_prediction/fransys.html#diag_rnn.output_to_hidden',
                                                                                                    'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN_raw': ( '04_prediction/fransys.html#diag_rnn_raw',
                                                                                       'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN_raw.__init__': ( '04_prediction/fransys.html#diag_rnn_raw.__init__',
                                                                                                'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN_raw.forward': ( '04_prediction/fransys.html#diag_rnn_raw.forward',
                                                                                               'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_RNN_raw.output_to_hidden': ( '04_prediction/fransys.html#diag_rnn_raw.output_to_hidden',
                                                                                                        'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_TCN': ( '04_prediction/fransys.html#diag_tcn',
                                                                                   'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_TCN.__init__': ( '04_prediction/fransys.html#diag_tcn.__init__',
                                                                                            'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_TCN.forward': ( '04_prediction/fransys.html#diag_tcn.forward',
                                                                                           'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.Diag_TCN.output_to_hidden': ( '04_prediction/fransys.html#diag_tcn.output_to_hidden',
                                                                                                    'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSys': ( '04_prediction/fransys.html#fransys',
                                                                                  'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSys.__init__': ( '04_prediction/fransys.html#fransys.__init__',
                                                                                           'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSys.forward': ( '04_prediction/fransys.html#fransys.forward',
                                                                                          'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback': ( '04_prediction/fransys.html#fransyscallback',
                                                                                          'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback.__init__': ( '04_prediction/fransys.html#fransyscallback.__init__',
                                                                                                   'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback.after_loss': ( '04_prediction/fransys.html#fransyscallback.after_loss',
                                                                                                     'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback.before_batch': ( '04_prediction/fransys.html#fransyscallback.before_batch',
                                                                                                       'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback.clear': ( '04_prediction/fransys.html#fransyscallback.clear',
                                                                                                'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback.hook': ( '04_prediction/fransys.html#fransyscallback.hook',
                                                                                               'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback_variable_init': ( '04_prediction/fransys.html#fransyscallback_variable_init',
                                                                                                        'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback_variable_init.__init__': ( '04_prediction/fransys.html#fransyscallback_variable_init.__init__',
                                                                                                                 'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysCallback_variable_init.before_batch': ( '04_prediction/fransys.html#fransyscallback_variable_init.before_batch',
                                                                                                                     'tsfast/prediction/fransys.py'),
                                           'tsfast.prediction.fransys.FranSysLearner': ( '04_prediction/fransys.html#fransyslearner',
                                                                                         'tsfast/prediction/fransys.py')},
            'tsfast.quaternions': { 'tsfast.quaternions.HDF2Inclination': ('quaternions.html#hdf2inclination', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.HDF2Inclination._hdf_extract_sequence': ( 'quaternions.html#hdf2inclination._hdf_extract_sequence',
                                                                                                  'tsfast/quaternions.py'),
                                    'tsfast.quaternions.HDF2Quaternion': ('quaternions.html#hdf2quaternion', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.HDF2Quaternion._hdf_extract_sequence': ( 'quaternions.html#hdf2quaternion._hdf_extract_sequence',
                                                                                                 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.InclinationBlock': ('quaternions.html#inclinationblock', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.InclinationBlock.__init__': ( 'quaternions.html#inclinationblock.__init__',
                                                                                      'tsfast/quaternions.py'),
                                    'tsfast.quaternions.InclinationBlock.from_hdf': ( 'quaternions.html#inclinationblock.from_hdf',
                                                                                      'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionAugmentation': ( 'quaternions.html#quaternionaugmentation',
                                                                                   'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionAugmentation.__call__': ( 'quaternions.html#quaternionaugmentation.__call__',
                                                                                            'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionAugmentation.__init__': ( 'quaternions.html#quaternionaugmentation.__init__',
                                                                                            'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionAugmentation.encodes': ( 'quaternions.html#quaternionaugmentation.encodes',
                                                                                           'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionBlock': ('quaternions.html#quaternionblock', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionBlock.__init__': ( 'quaternions.html#quaternionblock.__init__',
                                                                                     'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionBlock.from_hdf': ( 'quaternions.html#quaternionblock.from_hdf',
                                                                                     'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionRegularizer': ( 'quaternions.html#quaternionregularizer',
                                                                                  'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionRegularizer.__init__': ( 'quaternions.html#quaternionregularizer.__init__',
                                                                                           'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionRegularizer.after_loss': ( 'quaternions.html#quaternionregularizer.after_loss',
                                                                                             'tsfast/quaternions.py'),
                                    'tsfast.quaternions.QuaternionRegularizer.hook': ( 'quaternions.html#quaternionregularizer.hook',
                                                                                       'tsfast/quaternions.py'),
                                    'tsfast.quaternions.Quaternion_ResamplingModel': ( 'quaternions.html#quaternion_resamplingmodel',
                                                                                       'tsfast/quaternions.py'),
                                    'tsfast.quaternions.Quaternion_ResamplingModel.__init__': ( 'quaternions.html#quaternion_resamplingmodel.__init__',
                                                                                                'tsfast/quaternions.py'),
                                    'tsfast.quaternions.Quaternion_ResamplingModel.forward': ( 'quaternions.html#quaternion_resamplingmodel.forward',
                                                                                               'tsfast/quaternions.py'),
                                    'tsfast.quaternions.TensorInclination': ('quaternions.html#tensorinclination', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.TensorQuaternionAngle': ( 'quaternions.html#tensorquaternionangle',
                                                                                  'tsfast/quaternions.py'),
                                    'tsfast.quaternions.TensorQuaternionInclination': ( 'quaternions.html#tensorquaternioninclination',
                                                                                        'tsfast/quaternions.py'),
                                    'tsfast.quaternions.abs_inclination': ('quaternions.html#abs_inclination', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.abs_rel_angle': ('quaternions.html#abs_rel_angle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.angle_loss': ('quaternions.html#angle_loss', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.angle_loss_opt': ('quaternions.html#angle_loss_opt', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.augmentation_groups': ( 'quaternions.html#augmentation_groups',
                                                                                'tsfast/quaternions.py'),
                                    'tsfast.quaternions.conjQuat': ('quaternions.html#conjquat', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.deg_rmse': ('quaternions.html#deg_rmse', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.diffQuat': ('quaternions.html#diffquat', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclinationAngle': ('quaternions.html#inclinationangle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclinationAngleAbs': ( 'quaternions.html#inclinationangleabs',
                                                                                'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclination_loss': ('quaternions.html#inclination_loss', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclination_loss_abs': ( 'quaternions.html#inclination_loss_abs',
                                                                                 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclination_loss_smooth': ( 'quaternions.html#inclination_loss_smooth',
                                                                                    'tsfast/quaternions.py'),
                                    'tsfast.quaternions.inclination_loss_squared': ( 'quaternions.html#inclination_loss_squared',
                                                                                     'tsfast/quaternions.py'),
                                    'tsfast.quaternions.mean_inclination_deg': ( 'quaternions.html#mean_inclination_deg',
                                                                                 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.mean_rel_angle_deg': ( 'quaternions.html#mean_rel_angle_deg',
                                                                               'tsfast/quaternions.py'),
                                    'tsfast.quaternions.ms_inclination': ('quaternions.html#ms_inclination', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.ms_rel_angle': ('quaternions.html#ms_rel_angle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.multiplyQuat': ('quaternions.html#multiplyquat', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.multiplyQuat_np': ('quaternions.html#multiplyquat_np', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.norm_quaternion': ('quaternions.html#norm_quaternion', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.pitchAngle': ('quaternions.html#pitchangle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.plot_quaternion_inclination': ( 'quaternions.html#plot_quaternion_inclination',
                                                                                        'tsfast/quaternions.py'),
                                    'tsfast.quaternions.plot_quaternion_rel_angle': ( 'quaternions.html#plot_quaternion_rel_angle',
                                                                                      'tsfast/quaternions.py'),
                                    'tsfast.quaternions.plot_scalar_inclination': ( 'quaternions.html#plot_scalar_inclination',
                                                                                    'tsfast/quaternions.py'),
                                    'tsfast.quaternions.quatFromAngleAxis': ('quaternions.html#quatfromangleaxis', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.quatFromAngleAxis_np': ( 'quaternions.html#quatfromangleaxis_np',
                                                                                 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.quatInterp': ('quaternions.html#quatinterp', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.quatInterp_np': ('quaternions.html#quatinterp_np', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rad2deg': ('quaternions.html#rad2deg', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rand_quat': ('quaternions.html#rand_quat', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.relativeAngle': ('quaternions.html#relativeangle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.relativeQuat': ('quaternions.html#relativequat', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.relativeQuat_np': ('quaternions.html#relativequat_np', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rms_inclination': ('quaternions.html#rms_inclination', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rms_inclination_deg': ( 'quaternions.html#rms_inclination_deg',
                                                                                'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rms_pitch_deg': ('quaternions.html#rms_pitch_deg', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rms_rel_angle_deg': ('quaternions.html#rms_rel_angle_deg', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rms_roll_deg': ('quaternions.html#rms_roll_deg', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rollAngle': ('quaternions.html#rollangle', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.rot_vec': ('quaternions.html#rot_vec', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.safe_acos': ('quaternions.html#safe_acos', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.safe_acos_double': ('quaternions.html#safe_acos_double', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.show_batch': ('quaternions.html#show_batch', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.show_results': ('quaternions.html#show_results', 'tsfast/quaternions.py'),
                                    'tsfast.quaternions.smooth_inclination': ( 'quaternions.html#smooth_inclination',
                                                                               'tsfast/quaternions.py')},
            'tsfast.spectogram': { 'tsfast.spectogram.Sequence2Spectrogram': ( 'spectogram.html#sequence2spectrogram',
                                                                               'tsfast/spectogram.py'),
                                   'tsfast.spectogram.Sequence2Spectrogram.__init__': ( 'spectogram.html#sequence2spectrogram.__init__',
                                                                                        'tsfast/spectogram.py'),
                                   'tsfast.spectogram.Sequence2Spectrogram.encodes': ( 'spectogram.html#sequence2spectrogram.encodes',
                                                                                       'tsfast/spectogram.py'),
                                   'tsfast.spectogram.Spectrogram': ('spectogram.html#spectrogram', 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.Spectrogram.__init__': ( 'spectogram.html#spectrogram.__init__',
                                                                               'tsfast/spectogram.py'),
                                   'tsfast.spectogram.Spectrogram.forward': ('spectogram.html#spectrogram.forward', 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.SpectrogramBlock': ('spectogram.html#spectrogramblock', 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.SpectrogramBlock.__init__': ( 'spectogram.html#spectrogramblock.__init__',
                                                                                    'tsfast/spectogram.py'),
                                   'tsfast.spectogram.SpectrogramBlock.from_hdf': ( 'spectogram.html#spectrogramblock.from_hdf',
                                                                                    'tsfast/spectogram.py'),
                                   'tsfast.spectogram.TensorSpectrogram': ('spectogram.html#tensorspectrogram', 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.TensorSpectrogram.show': ( 'spectogram.html#tensorspectrogram.show',
                                                                                 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.TensorSpectrogramInput': ( 'spectogram.html#tensorspectrograminput',
                                                                                 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.TensorSpectrogramOutput': ( 'spectogram.html#tensorspectrogramoutput',
                                                                                  'tsfast/spectogram.py'),
                                   'tsfast.spectogram.complex_norm': ('spectogram.html#complex_norm', 'tsfast/spectogram.py'),
                                   'tsfast.spectogram.spectrogram': ('spectogram.html#spectrogram', 'tsfast/spectogram.py')},
            'tsfast.tune': { 'tsfast.tune.CBRayReporter': ('tune.html#cbrayreporter', 'tsfast/tune.py'),
                             'tsfast.tune.CBRayReporter.after_epoch': ('tune.html#cbrayreporter.after_epoch', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer': ('tune.html#hpoptimizer', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.__init__': ('tune.html#hpoptimizer.__init__', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.best_model': ('tune.html#hpoptimizer.best_model', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.optimize': ('tune.html#hpoptimizer.optimize', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.optimize_pbt': ('tune.html#hpoptimizer.optimize_pbt', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.start_ray': ('tune.html#hpoptimizer.start_ray', 'tsfast/tune.py'),
                             'tsfast.tune.HPOptimizer.stop_ray': ('tune.html#hpoptimizer.stop_ray', 'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable': ('tune.html#learnertrainable', 'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable._export_model': ('tune.html#learnertrainable._export_model', 'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable.load_checkpoint': ( 'tune.html#learnertrainable.load_checkpoint',
                                                                               'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable.reset_config': ('tune.html#learnertrainable.reset_config', 'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable.save_checkpoint': ( 'tune.html#learnertrainable.save_checkpoint',
                                                                               'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable.setup': ('tune.html#learnertrainable.setup', 'tsfast/tune.py'),
                             'tsfast.tune.LearnerTrainable.step': ('tune.html#learnertrainable.step', 'tsfast/tune.py'),
                             'tsfast.tune.learner_optimize': ('tune.html#learner_optimize', 'tsfast/tune.py'),
                             'tsfast.tune.log_uniform': ('tune.html#log_uniform', 'tsfast/tune.py'),
                             'tsfast.tune.sample_config': ('tune.html#sample_config', 'tsfast/tune.py'),
                             'tsfast.tune.stop_shared_memory_managers': ('tune.html#stop_shared_memory_managers', 'tsfast/tune.py')}}}
